@@ -54,7 +54,7 @@ class HashJoinExecutor(JoinExecutor):
                     selects_valid = self.selects_validation()
 
                 if (not selects_valid):
-                    print("Some join columns are not selected!")
+                    raise Exception("Some join columns are not selected!")
                     return []
 
 
@@ -97,8 +97,19 @@ class HashJoinExecutor(JoinExecutor):
                     left_meta_rows = session.execute(check_leftcols_query)
                     self.join_metadata.add_table(left_table)
 
+                    # Checking is select used?
+                    select_from_command = False
+                    if (left_table in self.selected_cols):
+                        select_from_command = True
+                        left_meta_rows = list(self.selected_cols[left_table])
+
                     for row in left_meta_rows:
-                        column_name = row['column_name']
+                        column_name = None
+                        if (select_from_command):
+                            column_name = row
+                        else:
+                            column_name = row["column_name"]
+
                         self.join_metadata.add_one_column(left_table, column_name)
         
                 if (not left_table in self.table_query):
@@ -125,15 +136,26 @@ class HashJoinExecutor(JoinExecutor):
                     right_meta_rows = session.execute(check_rightcols_query)
                     self.join_metadata.add_table(right_table)
 
+                    # Checking is select used?
+                    select_from_command = False
+                    if (right_table in self.selected_cols):
+                        select_from_command = True
+                        right_meta_rows = list(self.selected_cols[right_table])
+
                     for row in right_meta_rows:
-                        column_name = row['column_name']
+                        column_name = None
+                        if (select_from_command):
+                            column_name = row
+                        else:
+                            column_name = row["column_name"]
+
                         self.join_metadata.add_one_column(right_table, column_name)
 
                 
                 table_cols = self.join_metadata.get_columns_of_table(right_table)
                 if not(join_column_right in table_cols):
                     # Throw error
-                    print("Join column is not in right-table")
+                    raise Exception("Join column is not in right-table")
 
 
                 # Build select query
@@ -706,6 +728,16 @@ class HashJoinExecutor(JoinExecutor):
 
                 # Left table is build table by default
                 is_build_table = True
+
+                if (left_key == None):
+                    if (join_type == "INNER"):
+                        continue
+                    elif (join_type == "LEFT_OUTER"):
+                        intermediate_result.add_row_to_left_nomatch(row)
+                    elif (join_type == "RIGHT_OUTER"):
+                        continue
+                    else :
+                        intermediate_result.add_row_to_left_nomatch(row)
                 
                 # Use all rows to build hash table
                 intermediate_result.add_row_to_intermediate(row, is_build_table)
@@ -721,6 +753,16 @@ class HashJoinExecutor(JoinExecutor):
                 # Right table is probe table by default
                 is_build_table = False
 
+                if (right_key == None):
+                    if (join_type == "INNER"):
+                        continue
+                    elif (join_type == "LEFT_OUTER"):
+                        continue
+                    elif (join_type == "RIGHT_OUTER"):
+                        intermediate_result.add_row_to_right_nomatch(row)
+                    else:
+                        intermediate_result.add_row_to_right_nomatch(row)
+
                 # Check if key exists in hash table. Action based on join type
                 is_key_exists = intermediate_result.is_key_in_hashtable(right_key)
 
@@ -756,6 +798,16 @@ class HashJoinExecutor(JoinExecutor):
                 dict_key = (join_column_right, right_table_name)
                 right_key = row[dict_key]
 
+                if (right_key == None):
+                    if (join_type == "INNER"):
+                        continue
+                    elif (join_type == "LEFT_OUTER"):
+                        continue
+                    elif (join_type == "RIGHT_OUTER"):
+                        intermediate_result.add_row_to_right_nomatch(row)
+                    else:
+                        intermediate_result.add_row_to_right_nomatch(row)
+
                 # Now that the tables have been swap, right table is now a Build table
                 is_build_table = True
 
@@ -772,6 +824,16 @@ class HashJoinExecutor(JoinExecutor):
                 # Left table is build table by default
                 is_build_table = False
 
+                if (left_key == None):
+                    if (join_type == "INNER"):
+                        continue
+                    elif (join_type == "LEFT_OUTER"):
+                        intermediate_result.add_row_to_left_nomatch(row)
+                    elif (join_type == "RIGHT_OUTER"):
+                        continue
+                    else :
+                        intermediate_result.add_row_to_left_nomatch(row)
+
                 # Check if key exists in hash table. Action based on join type
                 is_key_exists = intermediate_result.is_key_in_hashtable(left_key)
 
