@@ -75,7 +75,7 @@ class NestedJoinExecutor(JoinExecutor):
                     selects_valid = self.selects_validation()
 
                 if (not selects_valid):
-                    print("Some join columns are not selected!")
+                    raise Exception("Some join columns are not selected!")
                     return []
 
                 join_command = command
@@ -116,8 +116,19 @@ class NestedJoinExecutor(JoinExecutor):
                     left_meta_rows = session.execute(check_leftcols_query)
                     self.join_metadata.add_table(left_table)
 
+                    # Checking is select used?
+                    select_from_command = False
+                    if (left_table in self.selected_cols):
+                        select_from_command = True
+                        left_meta_rows = list(self.selected_cols[left_table])
+
                     for row in left_meta_rows:
-                        column_name = row['column_name']
+                        column_name = None
+                        if (select_from_command):
+                            column_name = row
+                        else:
+                            column_name = row["column_name"]
+
                         self.join_metadata.add_one_column(left_table, column_name)
         
                 if (not left_table in self.table_query):
@@ -144,15 +155,26 @@ class NestedJoinExecutor(JoinExecutor):
                     right_meta_rows = session.execute(check_rightcols_query)
                     self.join_metadata.add_table(right_table)
 
+                    # Checking is select used?
+                    select_from_command = False
+                    if (right_table in self.selected_cols):
+                        select_from_command = True
+                        right_meta_rows = list(self.selected_cols[right_table])
+
                     for row in right_meta_rows:
-                        column_name = row['column_name']
+                        column_name = None
+                        if (select_from_command):
+                            column_name = row
+                        else:
+                            column_name = row["column_name"]
+                        
                         self.join_metadata.add_one_column(right_table, column_name)
 
                 # No join columns exception here
                 table_cols = self.join_metadata.get_columns_of_table(right_table)
                 if not(join_column_right in table_cols):
                     # Throw error
-                    print("Join column is not in right-table")
+                    raise Exception("Join column is not in right-table")
 
 
                 # Build select query
@@ -1027,8 +1049,6 @@ class NestedJoinExecutor(JoinExecutor):
 
                 partition_file.close()
 
-            # TODO: Delete all tmpfiles after join operation
-
         
         else : # Final result is in memory (self.current_result), return immediately
             
